// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License. See License.txt in the project root for
// license information.
// 
// Code generated by Microsoft (R) AutoRest Code Generator.
// Changes may cause incorrect behavior and will be lost if the code is
// regenerated.

namespace Microsoft.Azure.Search.Models
{
    using System;
    using System.Linq;
    using System.Collections.Generic;
    using Newtonsoft.Json;
    using Microsoft.Rest;
    using Microsoft.Rest.Serialization;
    using Microsoft.Rest.Azure;

    /// <summary>
    /// Abstract base class for functions that can modify document scores
    /// during ranking.
    /// <see href="https://msdn.microsoft.com/library/azure/dn798928.aspx" />
    /// </summary>
    public partial class ScoringFunction
    {
        /// <summary>
        /// Initializes a new instance of the ScoringFunction class.
        /// </summary>
        public ScoringFunction() { }

        /// <summary>
        /// Initializes a new instance of the ScoringFunction class.
        /// </summary>
        public ScoringFunction(string fieldName, double boost, ScoringFunctionInterpolation? interpolation = default(ScoringFunctionInterpolation?))
        {
            FieldName = fieldName;
            Boost = boost;
            Interpolation = interpolation;
        }

        /// <summary>
        /// Gets or sets the name of the field used as input to the scoring
        /// function.
        /// </summary>
        [JsonProperty(PropertyName = "fieldName")]
        public string FieldName { get; set; }

        /// <summary>
        /// Gets or sets a multiplier for the raw score. Must be a positive
        /// number not equal to 1.0.
        /// </summary>
        [JsonProperty(PropertyName = "boost")]
        public double Boost { get; set; }

        /// <summary>
        /// Gets or sets a value indicating how boosting will be interpolated
        /// across document scores; defaults to "Linear". Possible values
        /// include: 'linear', 'constant', 'quadratic', 'logarithmic'
        /// </summary>
        [JsonProperty(PropertyName = "interpolation")]
        public ScoringFunctionInterpolation? Interpolation { get; set; }

        /// <summary>
        /// Validate the object.
        /// </summary>
        /// <exception cref="ValidationException">
        /// Thrown if validation fails
        /// </exception>
        public virtual void Validate()
        {
            if (FieldName == null)
            {
                throw new ValidationException(ValidationRules.CannotBeNull, "FieldName");
            }
        }
    }
}
